import 'dart:async';
import 'dart:io';
import 'dart:isolate';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:mzanalytics_mybmw/mzanalytics_mybmw.dart';

// ignore: camel_case_types
class Mzanalytics_bugService {
  Mzanalytics_bugService._();

  static const MethodChannel _channel =
      const MethodChannel('crazecoder/flutter_bugly');

  ///异常上报
  static void exceptionListener<T>(
    T callback(), {
    FlutterExceptionHandler handler, //异常捕捉，用于自定义打印异常
    String filterRegExp, //异常上报过滤正则，针对message
    bool debugUpload = false,
  }) {
    bool _isDebug = false;
    assert(_isDebug = true);
    Isolate.current.addErrorListener(new RawReceivePort((dynamic pair) {
      var isolateError = pair as List<dynamic>;
      var _error = isolateError.first;
      var _stackTrace = isolateError.last;
      Zone.current.handleUncaughtError(_error, _stackTrace);
    }).sendPort);
    // This creates a [Zone] that contains the Flutter application and stablishes
    // an error handler that captures errors and reports them.
    //
    // Using a zone makes sure that as many errors as possible are captured,
    // including those thrown from [Timer]s, microtasks, I/O, and those forwarded
    // from the `FlutterError` handler.
    //
    // More about zones:
    //
    // - https://api.dartlang.org/stable/1.24.2/dart-async/Zone-class.html
    // - https://www.dartlang.org/articles/libraries/zones
    runZonedGuarded<Future<Null>>(() async {
      callback();
    }, (error, stackTrace) {
      _filterAndUploadException(
        debugUpload,
        _isDebug,
        handler,
        filterRegExp,
        FlutterErrorDetails(exception: error, stack: stackTrace),
      );
    });
    // This captures errors reported by the Flutter framework.
    FlutterError.onError = (FlutterErrorDetails details) {
      dynamic e = details.exception;
      String errorMessage;
      try {
        errorMessage = e.message;
      } catch (e) {
        errorMessage = '';
      }
      String stack = details.stack.toString();
      if ((stack is String) == true) {
        stack = stack.substring(0, stack.length > 5000 ? 5000 : stack.length);
      }

      Map exceptionMap = {
        "name": e.toString(),
        "reason": errorMessage,
        "stack": stack,
        "customerror": "1"
      };

      MzanalyticsMybmw.trackException(exceptionMap);
      Zone.current.handleUncaughtError(details.exception, details.stack);
    };
  }

  static void _filterAndUploadException(
    debugUpload,
    _isDebug,
    handler,
    filterRegExp,
    FlutterErrorDetails details,
  ) {}

  static bool _filterException(
      bool debugUpload,
      bool _isDebug,
      FlutterExceptionHandler handler,
      String filterRegExp,
      FlutterErrorDetails details) {
    //默认debug下打印异常，不上传异常
    if (!debugUpload && _isDebug) {
      handler == null
          ? FlutterError.dumpErrorToConsole(details)
          : handler(details);
      return true;
    }
    //异常过滤
    if (filterRegExp != null) {
      RegExp reg = new RegExp(filterRegExp);
      Iterable<Match> matches = reg.allMatches(details.exception.toString());
      if (matches.length > 0) {
        return true;
      }
    }
    return false;
  }

  ///上报自定义异常信息，data为文本附件
  ///Android 错误分析=>跟踪数据=>extraMessage.txt
  ///iOS 错误分析=>跟踪数据=>crash_attach.log

}
